@@ -9,19 +9,10 @@
 // except according to those terms.
 
 #![sanitizer_runtime]
+#![feature(nll)]
 #![feature(sanitizer_runtime)]
-#![feature(alloc_system)]
-#![feature(allocator_api)]
-#![feature(global_allocator)]
 #![feature(staged_api)]
 #![no_std]
 #![unstable(feature = "sanitizer_runtime_lib",
             reason = "internal implementation detail of sanitizers",
             issue = "0")]
-
-extern crate alloc_system;
-
-use alloc_system::System;
-
-#[global_allocator]
-static ALLOC: System = System;
